.include "Makros.asm"
.include "SRAM.asm"
.include "Konstanten.asm"
.include "Register.asm"
.include "Interrupts.asm"
;Copyright Christian Knll / Heidelbergerstr. 6 / 74746 Hpfingen / Deutschland
;Der Autor bernimmt keine Haftung fr ggf. direkt oder indirekt durch die Nutzung entstehende Vermgens- Sach- oder Personenschden.
;Kommerzielle Nutzung und Anpassungen gestattet.
; 
;
;Beschreibung des TWI / I2C Datentransfers zwischen Steuerung und Handrad:
;
;Steuerung Richtung Handrad:
;	Jedes Datenframe besteht aus 9 Bytes:
;		Byte 1 = Befehl bzw. Modus in den das Handrad gehen soll.
;		Byte 2-9 = Daten (Altlast ohne Funktion - einfach einlesen und ignorieren)
;
;	Gltige Befehle fr Byte 1 sind:
;		1:	Normale bertragung der Handrad-Daten.
;		3:	Aufforderung zur Identifizierung: Das Handrad muss mit "Handrad 0001" antworten - sonst wird es nicht von der Steuerung akzeptiert.
;		4:	Challenge: berbleibsel eines Kopierschutzes. Das Handrad muss mit Hexadezimal "3B59E82AE9B1BED800000000" antworten - sonst wird es zumindest von lteren Estlcam Versionen abgelehnt.
;		5:	Abfrage Seriennummer Atmega328PB: Das Handrad sollte die Seriennummer des Chips bertragen (10 Byte + 2 Bytes "0" zur Vervollstndigung des Datenframes), darf allerdings auch mit Fantasiedaten antworten.
;	Der Befehl bzw. Modus bleibt aktiv bis ein anderer Befehl kommt. Kommt z.b. der Befehl zur Identifizierung muss das Handrad solange damit antworten bis explizit ein anderer Befehl kommt.
;
;
;Handrad Richtung Steuerung:
;	Jedes Datenframe besteht aus 15 Bytes:
;	Bytes 1-12 = Daten
;	Byte 13	= Modus / Dieses Byte muss dem aktuellen Modus bzw. ausgefhrten Befehl entsprechen.
;	Byte 14	= Frame	/ mit jeder bertragung muss dieses Byte um 1 hochgezhlt werden. ndert sich dieses Byte nicht mehr geht die Steuerung von einem Problem mit dem Handrad aus, stoppt die Bewegung und versucht einen Reset der Kommunikation.
;	Byte 15 = Prfsumme / Frames deren Prfsumme inkorrekt ist werden verworfen um keine ungewollten Aktionen auszulsen.
;
;	Berechnung der Prfsumme:
;		Prfsumme zu Beginn des Datenframes auf 0 setzen.
;		Bytes 1 bis 14 mit XOR verknften und nach jedem Datenbyte Prfsumme um 1 erhhen.
;
;	Datenframe Modus 1:
;		Byte 1:	255 (Altlast - muss 255 zurckgeben, sonst ggf. Probleme!)
;		Byte 2: 
;			Bit 0 = Start / Stopp Taste Programmstart (1 = nicht gedrckt / 0 = gedrckt)
;			Bit 1 = Start / Stopp Taste Frsmotor (1 = nicht gedrckt / 0 = gedrckt)
;			Bit 2 = 1 (sonst ggf. Probleme!)
;			Bit 3 = "OK" Taste  (1 = nicht gedrckt / 0 = gedrckt)
;			Bit 4 bis 7 = 1 (sonst ggf. Probleme!)
;		Byte 3: Low Byte Vorschub Poti
;		Byte 4: High Byte Vorschub Poti
;		Byte 5: Low Byte Drehzahl Poti
;		Byte 6: High Byte Drehzahl Poti
;		Byte 7: Low Byte X Joystick
;		Byte 8: High Byte X Joystick
;		Byte 9: Low Byte Y Joystick
;		Byte 10: High Byte Y Joystick
;		Byte 11: Low Byte Z Joystick
;		Byte 12: High Byte Z Joystick
;

;Reset-------------------------------------------------------------------------------------------------------
Reset:
	st_w 	SP,RAMEND
			Clear_RAM			;Register und Ram auf 0 setzen...
	
	st_v	Inputs_8,255		;Eingnge alle inaktiv...
	st_v	Inputs_16,255
	
	Set_TWI_Mode 1				;TWI Modus 1

	st_v	DDRB,0b00000001		;Zugriffe auf DDR sollten zur Sicherheit nie ber st_0 laufen - falls das Register durch welchen Grund auch immer einen falschen Wert bekommen sollte wird sonst der Port falsch gesetzt!
	st_v	PortB,0b00000001
	st_v	DDRC,0
	st_0	PortC
	st_v	DDRD,0
	st_v	PortD,0b10100001   ;D0 Prog Start / D5 Spindle Start / D7 OK
	st_v	DDRE,0
	st_0	PortE
	
	st_v 	ADMUX,ADC_CH0		;ADC
	st_v 	ADCSRA,ADC_Start

	st_v	TWAR,TWI_Addr		;TWI
	st_v	TWCR,TWI_Enable			

	Init_Watchdog_250ms

	st_v	TCCR1B,0b00000100		;16 Bit Timer 1 / Prescaler 256 / 1Hz
	st_v	TIMSK1,0b00000001		;Overflow Interrupt
sei	

;Main--------------------------------------------------------------------------------------------------------
Main:	
		st_v	DDRB,0b00000001		;Zur Sicherheit...
		st_v	DDRC,0
		st_v	DDRD,0
		st_v	DDRE,0
		
		ld_t1		PinD
		ldi			t2,255			;t2 auf 255 - alles ausmaskieren...
		MoveBits	t1,0,t2,0		;Start CNC
		MoveBits	t1,5,t2,1		;Start Spindel
		MoveBits	t1,7,t2,3 		;OK		
		or    		Filter,t2
		subi  		Input_counter,32	brne Inputs_End
			sts		Inputs_16,Filter
			clr		Filter
	Inputs_End:
		.include	"ADC.asm"
	TWI_Check:
		ld_t1		TWCR
		sbrc		t1,TWINT	rcall TWI_Event
rjmp Main

;------------------------------------------------------------------------------------------------------------
;Schreibtransfer kann entweder vom Slave durch Besttigung ohne TWEA Bit beendet werden -> Anschlieend Statuscode 0xC8
;Oder durch den Master durch Stop oder wiederholten Start...
TWI_Event:			wdr
					st_0	TCCR1B								;LED Blinken aus...
					sbi		LED_Port,LED						;LED an...
					ld_t1 	TWSR								;Statusregister einlesen
					andi 	t1, 0xF8							;Prescaler rausmaskieren
					cpi 	t1, 0x60	breq TWI_Read_Start		;Slave Lesemodus: 		Adresse erhalten 		-> Nur besttigen...		
					cpi 	t1, 0x80  	breq TWI_Read			;Slave Lesemodus: 		Daten erhalten 			-> Daten lesen
					cpi 	t1, 0xA0	breq TWI_Read_End		;Slave Lesemodus: 		Stop erhalten 			-> Nur besttigen...
					cpi 	t1, 0xA8	breq TWI_Write_Start	;Slave Schreibmodus:	Adresse erhalten 		-> bertragung des 1. Bytes starten...
					cpi 	t1, 0xB8	breq TWI_Write			;Slave Schreibmodus:	Daten gesendet 			-> Daten senden...
					cpi		t1,	0xC0	breq TWI_Write_End		;Slave Schreibmodus:	Daten senden beendet...	-> Nur besttigen...			
;--------------------------------------------------------------------------------------------------------------------------------
	TWI_Error:		rcall		Reset_TWI_Bus						;Alle anderen Statuscodes = Fehler
					rjmp		TWI_End
;--------------------------------------------------------------------------------------------------------------------------------
	TWI_Read_Start:	ld_Y		TWI_Rx_Buffer
					rjmp 		TWI_Confirm
	;-----------------------
	TWI_Read:		ldi			YH,high(TWI_Rx_Buffer)				;Zur Sicherheit damit falls etwas schief gelaufen ist keine kritischen Register getroffen werden knnen...
					ld_t1		TWDR					
					st			Y+,t1					
					rjmp		TWI_Confirm					
	;-----------------------
	TWI_Read_End:	rcall		TWI_Data_Complete	
					rjmp 		TWI_Confirm
;--------------------------------------------------------------------------------------------------------------------------------
	TWI_Write_Start:
					clr			Checksum							;Prfsumme zurcksetzen...
					cpi			Modus,1 brne TWI_Write_St			;Falls Modus = 1 dann Zwischenspeicher in TWI Block kopieren, ansonsten direkt bertragung der Daten im TWI Block starten...
					rcall		Buffercopy

	TWI_Write_St:	ld_Y		TWI_Buffer
					Inc_Addr	t1,TWI_Frame						;TWI Frame Nummer um 1 erhhen...
															
	;-----------------------
	TWI_Write:		ld			t1,Y+								;Datenbyte holen
					st_t1		TWDR								;Datenbyte senden
					eor			Checksum,t1							;Prfsumme berechnen
					inc  		Checksum				
					sts			TWI_Checksum,Checksum				;Prfsumme sichern...
					rjmp		TWI_Confirm
	;-----------------------
	TWI_Write_End:	rjmp 		TWI_Confirm
;--------------------------------------------------------------------------------------------------------------------------------
TWI_Confirm:		st_v 		TWCR,TWI_Continue
TWI_End:			ret

;------------------------------------------------------------------------------------------------------------
TWI_Data_Complete:
					ld_t1		TWI_Rx_Buffer
					lsl			t1										;Befehlsnummer mit 2 multiplizieren, da wir jmp anstatt rjmp verwenden und die Befehle damit doppelt so lange sind...
					andi		t1,0b00001110							;Ungltige Befehle verhindern
					ld_Z   		Befehlstabelle
					add   		ZL,t1
					adc   		ZH,r_zero
					ijmp
	.include "Befehle.asm"

;-------------------------------------------------------------------------------------------------------
Buffercopy:
	st_v	In_8,255

	ld_t1	Inputs_16
	st_t1	In_16

	ld_t1	CH0L
	COM		t1
	st_t1	CHSL
	ld_t1	CH0H
	COM		t1
	st_t1	CHSH

	ld_t1	CH1L
	st_t1	CHZL
	ld_t1	CH1H
	st_t1	CHZH

	ld_t1	CH2L
	st_t1	CHYL
	ld_t1	CH2H
	st_t1	CHYH

	ld_t1	CH3L
	st_t1	CHXL
	ld_t1	CH3H
	st_t1	CHXH

	ld_t1	CH4L
	COM		t1
	st_t1	CHFL
	ld_t1	CH4H
	COM		t1
	st_t1	CHFH
ret

;-------------------------------------------------------------------------------------------------------
Challenge_Start:	.db 0x3B, 0x59, 0xE8, 0x2A, 0xE9, 0xB1, 0xBE, 0xD8, 0x00, 0x00, 0x00, 0x00					
Challenge_End:
ID_Start:			.db "Handrad 0001"
ID_End:


